use std::fmt::Display;

#[derive(sqlx::Type)]
#[sqlx(transparent)]
pub struct UserId(String);

impl UserId {
    pub fn new(name: &str, server_name: &str) -> anyhow::Result<Self> {
        let user_id = Self(format!("@{name}:{server_name}"));

        user_id.is_valid()?;

        Ok(user_id)
    }

    fn is_valid(&self) -> anyhow::Result<()> {
        (self.0.len() <= 255)
            .then(|| ())
            .ok_or(UserIdError::TooLong(self.0.len()))?;

        Ok(())
    }
}

impl Display for UserId {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.0)
    }
}

#[derive(Debug, thiserror::Error)]
pub enum UserIdError {
    #[error("UserId too long {0} (expected < 255)")]
    TooLong(usize),
    #[error("Invalid UserId given")]
    Invalid,
}
